#
# This file was generated using xslt from its XML file
#
# Copyright 2009, Associated Universities Inc., Washington DC
#
import sys
import os
from  casac import *
import string
from taskinit import casalog
from taskinit import xmlpath
#from taskmanager import tm
import task_sdaverage
def sdaverage(infile='', antenna=0, field='', spw='', scan='', pol='', timeaverage=False, tweight='tintsys', scanaverage=False, averageall=False, polaverage=False, pweight='tsys', kernel='', kwidth=5, chanwidth='5', verify=False, plotlevel=0, outfile='', outform='ASAP', overwrite=False):

        """ASAP SD task: averaging and smoothing of spectra

Keyword arguments:
infile -- name of input SD dataset
antenna -- select an antenna name or ID
        default: 0
        example: 'PM03'
        NOTE this parameter is effective only for MS input
field -- select data by field IDs and names
        default: '' (use all fields)
        example: field='3C2*' (all names starting with 3C2)
                 field='0,4,5~7' (field IDs 0,4,5,6,7)
                 field='0,3C273' (field ID 0 or field named 3C273)
        this selection is in addition to the other selections to data
spw -- select data by IF IDs (spectral windows)
       NOTE this task only supports IF ID selction and ignores channel
       selection.
        default: '' (use all IFs and channels)
        example: spw='3,5,7' (IF IDs 3,5,7; all channels)
                 spw='<2' (IF IDs less than 2, i.e., 0,1; all channels)
                 spw='30~45GHz' (IF IDs with the center frequencies in range 30-45GHz; all channels)
        this selection is in addition to the other selections to data
scan -- select data by scan numbers
        default: '' (use all scans)
        example: scan='21~23' (scan IDs 21,22,23)
        this selection is in addition to the other selections to data
pol -- select data by polarization IDs
        default: '' (use all polarizations)
        example: pol='0,1' (polarization IDs 0,1)
        this selection is in addition to the other selections to data
timeaverage -- average spectra over time
        options: (bool) True, False
        default: False

    >>>timeaverage expandable parameter
        tweight -- weighting for time averaging
                options: 'var'   (1/var(spec) weighted)
                         'tsys'  (1/Tsys**2 weighted)
                         'tint'  (integration time weighted)
                         'tintsys'  (Tint/Tsys**2)
                         'median'  ( median averaging)
                default: 'tintsys'
        scanaverage -- average spectra within a scan number
                       when True, spectra are NOT averaged over
                       different scan numbers.
                options: (bool) True, False
                default: False
        averageall -- average multi-resolution spectra
                       spectra are averaged by referring 
                       their frequency coverage
                 default: False

polaverage -- average spectra over polarizations
        options: (bool) True, False
        default: False

    >>>polaverage expandable parameter
        pweight -- weighting for polarization averaging
                options: 'var'  (1/var(spec) weighted)
                         'tsys' (1/Tsys**2 weighted)
                default: 'tsys'

kernel -- type of spectral smoothing kernel
        options: '', 'hanning','gaussian','boxcar','regrid'
        default: '' (no smoothing)

    >>>kernel expandable parameter
        kwidth -- width of spectral smoothing kernel
                options: (int) in channels 
                default: 5
	        example: 5 or 10 seem to be popular for boxcar
	                 ignored for hanning (fixed at 5 chans)
                         (0 will turn off gaussian or boxcar)
        chanwidth -- channel width of regridded spectra
	         default: '5' (in channels)
	         example: '500MHz', '0.2km/s'
        verify -- interactively verify the results of smoothing for each
                  spectrum. When verify = True, for each input spectrum,
                  spectra before and after the operation are displayed
                  in a plot window. At the prompt there are four choices
                  of action: 'Y' (accept the operation and continue to
                  the next input spectrum), 'N' (reject the operation
                  and continue to the next input spectrum), 'A' (accept
                  the current operation and continue non-interactively),
                  and 'R' (reject the current operation and exit from
                  operation). 
                  Note that when the operation is rejected by 'N' or 'R',
                  no smoothing is done to the spectrum/spectra.
                options: (bool) True,False
                default: False

outfile -- name of output file
        default: '' (<infile>_sm)
outform -- output file format
        options: 'ASAP','MS2', 'ASCII','SDFITS'
        default: 'ASAP'
        NOTE the ASAP format is easiest for further sd
        processing; use MS2 for CASA imaging.
        If ASCII, then will append some stuff to
        the outfile name
overwrite -- overwrite the output file if already exists
        options: (bool) True,False
        default: False
        NOTE this parameter is ignored when outform='ASCII'
plotlevel -- control for plotting and summary of smoothing results
        options: (int) 0, 1, 2, and their negative counterparts
	default: 0 (no plotting)
	example: plotlevel=1; plot spectra before and after smoothing
                 plotlevel=2; additionally list data before and after operation.
                 plotlevel<0 as abs(plotlevel), e.g.
                 -1 => hardcopy of final plot (will be named
                 <outfile>_smspec.eps)

-------------------------------------
AVERAGING OF SPECTRA
-------------------------------------
Task sdaverage has two modes of averaging spectra, i.e., time and
polarization average.

When timeaverage=True, spectra are averaged over time for each IF
(spectral window), polarization, and beam, independently. Note that,
by default (scanaverage=False), timeaverage=True averages spectra
irrespective of scan IDs.
It is possible to average spectra separately for each scan ID by setting
a sub-parameter scanaverage=True.
For example, the combination of parameters: scan='0~2', timeaverage=True, and
scanaverage=False: averages spectra in scan ID 0 through 2 all together
                   to a spectrum,
scanaverage=True : averages spectra per scan ID and end up with three
                   spectra from scan 0, 1, and 2.

When polaverage=True, spectra are averaged over polarization for
each IF (spectral window) and beam. Note that, so far, time averaging is
automatically switched on when polaverage is set to True. This behavior
is not desirable and will be discarded in future.

-------
WARNING
-------
For the GBT raw SDFITS format data as input:
SDtasks are able to handle GBT raw SDFITS format data since the 
data filler is available. However, the functionality is not well 
tested yet, so that there may be unknown bugs.  

  
        """

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['infile'] = infile
        mytmp['antenna'] = antenna
        mytmp['field'] = field
        mytmp['spw'] = spw
        mytmp['scan'] = scan
        mytmp['pol'] = pol
        mytmp['timeaverage'] = timeaverage
        mytmp['tweight'] = tweight
        mytmp['scanaverage'] = scanaverage
        mytmp['averageall'] = averageall
        mytmp['polaverage'] = polaverage
        mytmp['pweight'] = pweight
        mytmp['kernel'] = kernel
        mytmp['kwidth'] = kwidth
        mytmp['chanwidth'] = chanwidth
        mytmp['verify'] = verify
        mytmp['plotlevel'] = plotlevel
        mytmp['outfile'] = outfile
        mytmp['outform'] = outform
        mytmp['overwrite'] = overwrite
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
	trec = casac.utils().torecord(pathname+'sdaverage.xml')

        casalog.origin('sdaverage')
        if trec.has_key('sdaverage') and casac.utils().verify(mytmp, trec['sdaverage']) :
	    result = task_sdaverage.sdaverage(infile, antenna, field, spw, scan, pol, timeaverage, tweight, scanaverage, averageall, polaverage, pweight, kernel, kwidth, chanwidth, verify, plotlevel, outfile, outform, overwrite)

	else :
	  result = False
        return result
