#
# This file was generated using xslt from its XML file
#
# Copyright 2014, Associated Universities Inc., Washington DC
#
import sys
import os
#from casac import *
import casac
import string
import time
import inspect
import gc
import numpy
from odict import odict
from types import * 
from task_sdaverage import sdaverage
class sdaverage_cli_:
    __name__ = "sdaverage"
    rkey = None
    i_am_a_casapy_task = None
    # The existence of the i_am_a_casapy_task attribute allows help()
    # (and other) to treat casapy tasks as a special case.

    def __init__(self) :
       self.__bases__ = (sdaverage_cli_,)
       self.__doc__ = self.__call__.__doc__

       self.parameters={'infile':None, 'antenna':None, 'field':None, 'spw':None, 'scan':None, 'pol':None, 'timeaverage':None, 'tweight':None, 'scanaverage':None, 'averageall':None, 'polaverage':None, 'pweight':None, 'kernel':None, 'kwidth':None, 'chanwidth':None, 'verify':None, 'plotlevel':None, 'outfile':None, 'outform':None, 'overwrite':None, }


    def result(self, key=None):
	    #### and add any that have completed...
	    return None


    def __call__(self, infile=None, antenna=None, field=None, spw=None, scan=None, pol=None, timeaverage=None, tweight=None, scanaverage=None, averageall=None, polaverage=None, pweight=None, kernel=None, kwidth=None, chanwidth=None, verify=None, plotlevel=None, outfile=None, outform=None, overwrite=None, ):

        """ASAP SD task: averaging and smoothing of spectra

	Detailed Description: 

Task sdaverage performs averaging in time/polarization and smoothing
of the single-dish spectra. When timeaverage=True, spectra are averaged
in time. Spectra within each scan ID are averaged when scanaverage=True.
When polaverage=True, spectra are averaged in polarization and time
(Note time averaging with polaverage=True would be discarded in future).
See examples in below for details of time/polarization average.
When kernel is specified (!=\'\'), each spectrum is smoothed by
convolving the kernel after averaging of spectra.

If you give multiple IFs (spectral windows) in spw, then your scantable
will have multiple IFs by default. Averaging of multi-resolution
(multi-IFs) spectra can be achieved by setting a sub-parameter in
timeaverage, averageall = True. It handles multi-IFs by merging IFs
which have overlaps in frequency coverages and assigning new IFs in
the output spectra.

Set plotlevel &gt;= 1 to plot spectrum before and after smoothing, and
verify=True to interactively select whether or not accept smoothing
results.
NOTE, so far, there is no mechanism to verify averaging of spectra in time
and/or polarization.

	Arguments :
		infile:	name of input SD dataset
		   Default Value: 

		antenna:	select an antenna name or ID, e.g. \'PM03\' (only effective for MS input)
		   Default Value: 0

		field:	select data by field IDs and names, e.g. \'3C2*\' (\'\'=all)
		   Default Value: 

		spw:	select data by IF IDs (spectral windows), e.g. \'3,5,7\' (\'\'=all)
		   Default Value: 

		scan:	select data by scan numbers, e.g. \'21~23\' (\'\'=all)
		   Default Value: 

		pol:	select data by polarization IDs, e.g. \'0,1\' (\'\'=all)
		   Default Value: 

		timeaverage:	average spectra over time [True, False] (see examples in help)
		   Default Value: False

		tweight:	weighting for time averaging
		   Default Value: tintsys
		   Allowed Values:
				var
				tsys
				tint
				tintsys
				median

		scanaverage:	average spectra within a scan number [True, False] (see examples in help)
		   Default Value: False

		averageall:	set True only when averaging spectra with different spectral resolutions
		   Default Value: False

		polaverage:	average spectra over polarizations [True, False]
		   Default Value: False

		pweight:	weighting for polarization averaging
		   Default Value: tsys
		   Allowed Values:
				var
				tsys

		kernel:	type of spectral smoothing kernel  (\'\'=no smoothing)
		   Default Value: 
		   Allowed Values:
				
				hanning
				gaussian
				boxcar
				regrid

		kwidth:	width of smoothing kernel in channels
		   Default Value: 5

		chanwidth:	width of regridded channels
		   Default Value: 5

		verify:	interactively verify the results of smoothing for each spectrum. [not available for kernel="regrid"]
		   Default Value: False

		plotlevel:	plot and summarize results (0=none). See description in help
		   Default Value: 0

		outfile:	name of output file (See a WARNING in help) 
		   Default Value: 

		outform:	output file format (See a WARNING in help)
		   Default Value: ASAP
		   Allowed Values:
				ASCII
				ascii
				MS
				ms
				MS2
				ms2
				SDFITS
				sdfits
				ASAP
				asap

		overwrite:	overwrite the output file if already exists
		   Default Value: False

	Returns: void

	Example :


Keyword arguments:
infile -- name of input SD dataset
antenna -- select an antenna name or ID
        default: 0
        example: 'PM03'
        NOTE this parameter is effective only for MS input
field -- select data by field IDs and names
        default: '' (use all fields)
        example: field='3C2*' (all names starting with 3C2)
                 field='0,4,5~7' (field IDs 0,4,5,6,7)
                 field='0,3C273' (field ID 0 or field named 3C273)
        this selection is in addition to the other selections to data
spw -- select data by IF IDs (spectral windows)
       NOTE this task only supports IF ID selction and ignores channel
       selection.
        default: '' (use all IFs and channels)
        example: spw='3,5,7' (IF IDs 3,5,7; all channels)
                 spw='<2' (IF IDs less than 2, i.e., 0,1; all channels)
                 spw='30~45GHz' (IF IDs with the center frequencies in range 30-45GHz; all channels)
        this selection is in addition to the other selections to data
scan -- select data by scan numbers
        default: '' (use all scans)
        example: scan='21~23' (scan IDs 21,22,23)
        this selection is in addition to the other selections to data
pol -- select data by polarization IDs
        default: '' (use all polarizations)
        example: pol='0,1' (polarization IDs 0,1)
        this selection is in addition to the other selections to data
timeaverage -- average spectra over time
        options: (bool) True, False
        default: False

    >>>timeaverage expandable parameter
        tweight -- weighting for time averaging
                options: 'var'   (1/var(spec) weighted)
                         'tsys'  (1/Tsys**2 weighted)
                         'tint'  (integration time weighted)
                         'tintsys'  (Tint/Tsys**2)
                         'median'  ( median averaging)
                default: 'tintsys'
        scanaverage -- average spectra within a scan number
                       when True, spectra are NOT averaged over
                       different scan numbers.
                options: (bool) True, False
                default: False
        averageall -- average multi-resolution spectra
                       spectra are averaged by referring 
                       their frequency coverage
                 default: False

polaverage -- average spectra over polarizations
        options: (bool) True, False
        default: False

    >>>polaverage expandable parameter
        pweight -- weighting for polarization averaging
                options: 'var'  (1/var(spec) weighted)
                         'tsys' (1/Tsys**2 weighted)
                default: 'tsys'

kernel -- type of spectral smoothing kernel
        options: '', 'hanning','gaussian','boxcar','regrid'
        default: '' (no smoothing)

    >>>kernel expandable parameter
        kwidth -- width of spectral smoothing kernel
                options: (int) in channels 
                default: 5
	        example: 5 or 10 seem to be popular for boxcar
	                 ignored for hanning (fixed at 5 chans)
                         (0 will turn off gaussian or boxcar)
        chanwidth -- channel width of regridded spectra
	         default: '5' (in channels)
	         example: '500MHz', '0.2km/s'
        verify -- interactively verify the results of smoothing for each
                  spectrum. When verify = True, for each input spectrum,
                  spectra before and after the operation are displayed
                  in a plot window. At the prompt there are four choices
                  of action: 'Y' (accept the operation and continue to
                  the next input spectrum), 'N' (reject the operation
                  and continue to the next input spectrum), 'A' (accept
                  the current operation and continue non-interactively),
                  and 'R' (reject the current operation and exit from
                  operation). 
                  Note that when the operation is rejected by 'N' or 'R',
                  no smoothing is done to the spectrum/spectra.
                options: (bool) True,False
                default: False

outfile -- name of output file
        default: '' (<infile>_sm)
outform -- output file format
        options: 'ASAP','MS2', 'ASCII','SDFITS'
        default: 'ASAP'
        NOTE the ASAP format is easiest for further sd
        processing; use MS2 for CASA imaging.
        If ASCII, then will append some stuff to
        the outfile name
overwrite -- overwrite the output file if already exists
        options: (bool) True,False
        default: False
        NOTE this parameter is ignored when outform='ASCII'
plotlevel -- control for plotting and summary of smoothing results
        options: (int) 0, 1, 2, and their negative counterparts
	default: 0 (no plotting)
	example: plotlevel=1; plot spectra before and after smoothing
                 plotlevel=2; additionally list data before and after operation.
                 plotlevel<0 as abs(plotlevel), e.g.
                 -1 => hardcopy of final plot (will be named
                 <outfile>_smspec.eps)

-------------------------------------
AVERAGING OF SPECTRA
-------------------------------------
Task sdaverage has two modes of averaging spectra, i.e., time and
polarization average.

When timeaverage=True, spectra are averaged over time for each IF
(spectral window), polarization, and beam, independently. Note that,
by default (scanaverage=False), timeaverage=True averages spectra
irrespective of scan IDs.
It is possible to average spectra separately for each scan ID by setting
a sub-parameter scanaverage=True.
For example, the combination of parameters: scan='0~2', timeaverage=True, and
scanaverage=False: averages spectra in scan ID 0 through 2 all together
                   to a spectrum,
scanaverage=True : averages spectra per scan ID and end up with three
                   spectra from scan 0, 1, and 2.

When polaverage=True, spectra are averaged over polarization for
each IF (spectral window) and beam. Note that, so far, time averaging is
automatically switched on when polaverage is set to True. This behavior
is not desirable and will be discarded in future.

-------
WARNING
-------
For the GBT raw SDFITS format data as input:
SDtasks are able to handle GBT raw SDFITS format data since the 
data filler is available. However, the functionality is not well 
tested yet, so that there may be unknown bugs.  

  
        """
	if not hasattr(self, "__globals__") or self.__globals__ == None :
           self.__globals__=sys._getframe(len(inspect.stack())-1).f_globals
	#casac = self.__globals__['casac']
	casalog = self.__globals__['casalog']
	casa = self.__globals__['casa']
	#casalog = casac.casac.logsink()
        self.__globals__['__last_task'] = 'sdaverage'
        self.__globals__['taskname'] = 'sdaverage'
        ###
        self.__globals__['update_params'](func=self.__globals__['taskname'],printtext=False,ipython_globals=self.__globals__)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
        if type(self.__call__.func_defaults) is NoneType:
            function_signature_defaults={}
	else:
	    function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames[1:],self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       self.parameters[key] = keyVal
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
                      if len(keyVal) > 0 :
		         exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')
		      else :
		         exec('myparams[key] = ' + key + ' = {}')
	 
        else :
            print ''

            myparams['infile'] = infile = self.parameters['infile']
            myparams['antenna'] = antenna = self.parameters['antenna']
            myparams['field'] = field = self.parameters['field']
            myparams['spw'] = spw = self.parameters['spw']
            myparams['scan'] = scan = self.parameters['scan']
            myparams['pol'] = pol = self.parameters['pol']
            myparams['timeaverage'] = timeaverage = self.parameters['timeaverage']
            myparams['tweight'] = tweight = self.parameters['tweight']
            myparams['scanaverage'] = scanaverage = self.parameters['scanaverage']
            myparams['averageall'] = averageall = self.parameters['averageall']
            myparams['polaverage'] = polaverage = self.parameters['polaverage']
            myparams['pweight'] = pweight = self.parameters['pweight']
            myparams['kernel'] = kernel = self.parameters['kernel']
            myparams['kwidth'] = kwidth = self.parameters['kwidth']
            myparams['chanwidth'] = chanwidth = self.parameters['chanwidth']
            myparams['verify'] = verify = self.parameters['verify']
            myparams['plotlevel'] = plotlevel = self.parameters['plotlevel']
            myparams['outfile'] = outfile = self.parameters['outfile']
            myparams['outform'] = outform = self.parameters['outform']
            myparams['overwrite'] = overwrite = self.parameters['overwrite']


	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['infile'] = infile
        mytmp['antenna'] = antenna
        mytmp['field'] = field
        mytmp['spw'] = spw
        mytmp['scan'] = scan
        mytmp['pol'] = pol
        mytmp['timeaverage'] = timeaverage
        mytmp['tweight'] = tweight
        mytmp['scanaverage'] = scanaverage
        mytmp['averageall'] = averageall
        mytmp['polaverage'] = polaverage
        mytmp['pweight'] = pweight
        mytmp['kernel'] = kernel
        mytmp['kwidth'] = kwidth
        mytmp['chanwidth'] = chanwidth
        mytmp['verify'] = verify
        mytmp['plotlevel'] = plotlevel
        mytmp['outfile'] = outfile
        mytmp['outform'] = outform
        mytmp['overwrite'] = overwrite
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
	trec = casac.casac.utils().torecord(pathname+'sdaverage.xml')

        casalog.origin('sdaverage')
	try :
          #if not trec.has_key('sdaverage') or not casac.casac.utils().verify(mytmp, trec['sdaverage']) :
	    #return False

          casac.casac.utils().verify(mytmp, trec['sdaverage'], True)
          scriptstr=['']
          saveinputs = self.__globals__['saveinputs']
          if type(self.__call__.func_defaults) is NoneType:
              saveinputs=''
          else:
              saveinputs('sdaverage', 'sdaverage.last', myparams, self.__globals__,scriptstr=scriptstr)
          tname = 'sdaverage'
          spaces = ' '*(18-len(tname))
          casalog.post('\n##########################################'+
                       '\n##### Begin Task: ' + tname + spaces + ' #####')
          if type(self.__call__.func_defaults) is NoneType:
              casalog.post(scriptstr[0]+'\n', 'INFO')
          else :
              casalog.post(scriptstr[1][1:]+'\n', 'INFO')
          result = sdaverage(infile, antenna, field, spw, scan, pol, timeaverage, tweight, scanaverage, averageall, polaverage, pweight, kernel, kwidth, chanwidth, verify, plotlevel, outfile, outform, overwrite)
          casalog.post('##### End Task: ' + tname + '  ' + spaces + ' #####'+
                       '\n##########################################')

	except Exception, instance:
          if(self.__globals__.has_key('__rethrow_casa_exceptions') and self.__globals__['__rethrow_casa_exceptions']) :
             raise
          else :
             #print '**** Error **** ',instance
	     tname = 'sdaverage'
             casalog.post('An error occurred running task '+tname+'.', 'ERROR')
             pass

        gc.collect()
        return result
#
#
#
    def paramgui(self, useGlobals=True, ipython_globals=None):
        """
        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
        """
        import paramgui
	if not hasattr(self, "__globals__") or self.__globals__ == None :
           self.__globals__=sys._getframe(len(inspect.stack())-1).f_globals

        if useGlobals:
	    if ipython_globals == None:
                myf=self.__globals__
            else:
                myf=ipython_globals

            paramgui.setGlobals(myf)
        else:
            paramgui.setGlobals({})

        paramgui.runTask('sdaverage', myf['_ip'])
        paramgui.setGlobals({})

#
#
#
    def defaults(self, param=None, ipython_globals=None, paramvalue=None, subparam=None):
	if not hasattr(self, "__globals__") or self.__globals__ == None :
           self.__globals__=sys._getframe(len(inspect.stack())-1).f_globals
        if ipython_globals == None:
            myf=self.__globals__
        else:
            myf=ipython_globals

        a = odict()
        a['infile']  = ''
        a['antenna']  = 0
        a['field']  = ''
        a['spw']  = ''
        a['scan']  = ''
        a['pol']  = ''
        a['timeaverage']  = False
        a['polaverage']  = False
        a['kernel']  = ''
        a['outfile']  = ''
        a['outform']  = 'ASAP'
        a['overwrite']  = False

        a['timeaverage'] = {
                    0:{'value':False}, 
                    1:odict([{'value':True}, {'tweight':'tintsys'}, {'scanaverage':False}, {'averageall':False}])}
        a['polaverage'] = {
                    0:{'value':False}, 
                    1:odict([{'value':True}, {'pweight':'tsys'}])}
        a['kernel'] = {
                    0:{'value':''}, 
                    1:odict([{'value':'hanning'}, {'verify':False}, {'plotlevel':0}]), 
                    2:odict([{'value':'gaussian'}, {'kwidth':5}, {'verify':False}, {'plotlevel':0}]), 
                    3:odict([{'value':'boxcar'}, {'kwidth':5}, {'verify':False}, {'plotlevel':0}]), 
                    4:odict([{'value':'regrid'}, {'chanwidth':'5'}, {'plotlevel':0}])}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
            if(paramvalue==None and subparam==None):
               if(a.has_key(param)):
                  return a[param]
               else:
                  return self.itsdefault(param)
            else:
               retval=a[param]
               if(type(a[param])==dict):
                  for k in range(len(a[param])):
                     valornotval='value'
                     if(a[param][k].has_key('notvalue')):
                        valornotval='notvalue'
                     if((a[param][k][valornotval])==paramvalue):
                        retval=a[param][k].copy()
                        retval.pop(valornotval)
                        if(subparam != None):
                           if(retval.has_key(subparam)):
                              retval=retval[subparam]
                           else:
                              retval=self.itsdefault(subparam)
		     else:
                        retval=self.itsdefault(subparam)
               return retval


#
#
    def check_params(self, param=None, value=None, ipython_globals=None):
      if ipython_globals == None:
          myf=self.__globals__
      else:
          myf=ipython_globals
#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value
#
#
    def description(self, key='sdaverage', subkey=None):
        desc={'sdaverage': 'ASAP SD task: averaging and smoothing of spectra',
               'infile': 'name of input SD dataset',
               'antenna': 'select an antenna name or ID, e.g. \'PM03\' (only effective for MS input)',
               'field': 'select data by field IDs and names, e.g. \'3C2*\' (\'\'=all)',
               'spw': 'select data by IF IDs (spectral windows), e.g. \'3,5,7\' (\'\'=all)',
               'scan': 'select data by scan numbers, e.g. \'21~23\' (\'\'=all)',
               'pol': 'select data by polarization IDs, e.g. \'0,1\' (\'\'=all)',
               'timeaverage': 'average spectra over time [True, False] (see examples in help)',
               'tweight': 'weighting for time averaging [\'tintsys\', \'tsys\', \'tint\', \'var\', or \'median\']',
               'scanaverage': 'average spectra within a scan number [True, False] (see examples in help)',
               'averageall': 'set True only when averaging spectra with different spectral resolutions',
               'polaverage': 'average spectra over polarizations [True, False]',
               'pweight': 'weighting for polarization averaging [\'tsys\' or \'var\']',
               'kernel': 'type of spectral smoothing kernel [\'hanning\', \'gaussian\', \'boxcar\', \'regrid\'] (\'\'=no smoothing)',
               'kwidth': 'width of smoothing kernel in channels',
               'chanwidth': 'width of regridded channels',
               'verify': 'interactively verify the results of smoothing for each spectrum. [not available for kernel="regrid"]',
               'plotlevel': 'plot and summarize results (0=none). See description in help',
               'outfile': 'name of output file (See a WARNING in help) ',
               'outform': 'output file format [\'ASAP\', \'MS2\', \'ASCII\', or \'SDFITS\']',
               'overwrite': 'overwrite the output file if already exists [True, False]',

              }

#
# Set subfields defaults if needed
#

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['infile']  = ''
        a['antenna']  = 0
        a['field']  = ''
        a['spw']  = ''
        a['scan']  = ''
        a['pol']  = ''
        a['timeaverage']  = False
        a['tweight']  = 'tintsys'
        a['scanaverage']  = False
        a['averageall']  = False
        a['polaverage']  = False
        a['pweight']  = 'tsys'
        a['kernel']  = ''
        a['kwidth']  = 5
        a['chanwidth']  = '5'
        a['verify']  = False
        a['plotlevel']  = 0
        a['outfile']  = ''
        a['outform']  = 'ASAP'
        a['overwrite']  = False

        #a = sys._getframe(len(inspect.stack())-1).f_globals

        if self.parameters['timeaverage']  == True:
            a['tweight'] = 'tintsys'
            a['scanaverage'] = False
            a['averageall'] = False

        if self.parameters['polaverage']  == True:
            a['pweight'] = 'tsys'

        if self.parameters['kernel']  == 'hanning':
            a['verify'] = False
            a['plotlevel'] = 0

        if self.parameters['kernel']  == 'gaussian':
            a['kwidth'] = 5
            a['verify'] = False
            a['plotlevel'] = 0

        if self.parameters['kernel']  == 'boxcar':
            a['kwidth'] = 5
            a['verify'] = False
            a['plotlevel'] = 0

        if self.parameters['kernel']  == 'regrid':
            a['chanwidth'] = '5'
            a['plotlevel'] = 0

        if a.has_key(paramname) :
	      return a[paramname]
sdaverage_cli = sdaverage_cli_()
