#
# This file was generated using xslt from its XML file
#
# Copyright 2014, Associated Universities Inc., Washington DC
#
import sys
import os
#from casac import *
import casac
import string
import time
import inspect
import gc
import numpy
from odict import odict
from types import * 
from task_sdfit import sdfit
class sdfit_cli_:
    __name__ = "sdfit"
    rkey = None
    i_am_a_casapy_task = None
    # The existence of the i_am_a_casapy_task attribute allows help()
    # (and other) to treat casapy tasks as a special case.

    def __init__(self) :
       self.__bases__ = (sdfit_cli_,)
       self.__doc__ = self.__call__.__doc__

       self.parameters={'infile':None, 'antenna':None, 'fluxunit':None, 'telescopeparam':None, 'field':None, 'spw':None, 'restfreq':None, 'frame':None, 'doppler':None, 'scan':None, 'pol':None, 'timeaverage':None, 'tweight':None, 'scanaverage':None, 'polaverage':None, 'pweight':None, 'fitfunc':None, 'fitmode':None, 'nfit':None, 'thresh':None, 'min_nchan':None, 'avg_limit':None, 'box_size':None, 'edge':None, 'outfile':None, 'overwrite':None, 'plotlevel':None, }


    def result(self, key=None):
	    #### and add any that have completed...
	    return None


    def __call__(self, infile=None, antenna=None, fluxunit=None, telescopeparam=None, field=None, spw=None, restfreq=None, frame=None, doppler=None, scan=None, pol=None, timeaverage=None, tweight=None, scanaverage=None, polaverage=None, pweight=None, fitfunc=None, fitmode=None, nfit=None, thresh=None, min_nchan=None, avg_limit=None, box_size=None, edge=None, outfile=None, overwrite=None, plotlevel=None, ):

        """Fit a spectral line

	Detailed Description: 

Task sdfit is a basic line-fitter for single-dish spectra.
It assumes that the spectra have been calibrated in sdcal
or sdreduce.
  
	Arguments :
		infile:	name of input SD dataset
		   Default Value: 

		antenna:	select an antenna name or ID, e.g. \'PM03\' (only effective for MS input)
		   Default Value: 0

		fluxunit:	units of the flux (\'\'=current)
		   Default Value: 

		telescopeparam:	parameters of telescope for flux conversion (see examples in help)
		   Default Value: 

		field:	select data by field IDs and names, e.g. \'3C2*\' (\'\'=all)
		   Default Value: 

		spw:	select data by IF IDs (spectral windows), e.g. \'3,5,7\' (\'\'=all)
		   Default Value: 

		restfreq:	the rest frequency, e.g. \'1.41GHz\' (default unit: Hz) (see examples in help)
		   Default Value: 

		frame:	frequency reference frame (\'\'=current)
		   Default Value: 

		doppler:	doppler convention (\'\'=current). Effective only when spw selection is in velocity unit.
		   Default Value: 

		scan:	select data by scan numbers, e.g. \'21~23\' (\'\'=all)
		   Default Value: 

		pol:	select data by polarization IDs, e.g. \'0,1\' (\'\'=all)
		   Default Value: 

		timeaverage:	average spectra over time (see examples in help)
		   Default Value: False

		tweight:	weighting for time averaging
		   Default Value: tintsys
		   Allowed Values:
				var
				tsys
				tint
				tintsys
				median

		scanaverage:	average spectra within a scan number (see examples in help)
		   Default Value: False

		polaverage:	average spectra over polarizations
		   Default Value: False

		pweight:	weighting for polarization averaging
		   Default Value: tsys
		   Allowed Values:
				var
				tsys

		fitfunc:	function for fitting
		   Default Value: gauss
		   Allowed Values:
				gauss
				lorentz

		fitmode:	mode for fitting
		   Default Value: auto
		   Allowed Values:
				auto
				list
				interact

		nfit:	list of number of gaussian/lorentzian lines to fit in in maskline region (ignored when fitmode="auto")
		   Default Value: 

		thresh:	S/N threshold for linefinder
		   Default Value: 5.0

		min_nchan:	minimum number of consecutive channels for linefinder
		   Default Value: 3

		avg_limit:	channel averaging for broad lines
		   Default Value: 4

		box_size:	running mean box size
		   Default Value: 0.2

		edge:	channels to drop at beginning and end of spectrum
		   Default Value: 0

		outfile:	name of output file (See a WARNING in help)
		   Default Value: 

		overwrite:	overwrite the output file if already exists
		   Default Value: False

		plotlevel:	control for plotting of results (see examples in help)
		   Default Value: 0

	Returns: variant

	Example :

-----------------
Keyword arguments
-----------------
infile -- name of input SD dataset
antenna -- select an antenna name or ID
        default: 0
        example: 'PM03'
        NOTE this parameter is effective only for MS input
fluxunit -- units for line flux
        options: 'K','Jy',''
        default: '' (keep current fluxunit in data)
        WARNING: For GBT data, see description below.
    >>> fluxunit expandable parameter
        telescopeparam -- parameters of telescope for flux conversion
                options: (str) name or (list) list of gain info
                default: '' (none set)
                example: if telescopeparam='', it tries to get the telescope
                         name from the data.
                         Full antenna parameters (diameter,ap.eff.) known
                         to ASAP are
                         'ATPKSMB', 'ATPKSHOH', 'ATMOPRA', 'DSS-43',
                         'CEDUNA','HOBART'. For GBT, it fixes default fluxunit
                         to 'K' first then convert to a new fluxunit.
                         telescopeparam=[104.9,0.43] diameter(m), ap.eff.
                         telescopeparam=[0.743] gain in Jy/K
                         telescopeparam='FIX' to change default fluxunit
                         see description below
field -- select data by field IDs and names
        default: '' (use all fields)
        example: field='3C2*' (all names starting with 3C2)
                 field='0,4,5~7' (field IDs 0,4,5,6,7)
                 field='0,3C273' (field ID 0 or field named 3C273)
        this selection is in addition to the other selections to data
spw -- select data by IF IDs (spectral windows)/channels
        default: '' (use all IFs and channels)
        example: spw='3,5,7' (IF IDs 3,5,7; all channels)
                 spw='<2' (IF IDs less than 2, i.e., 0,1; all channels)
                 spw='30~45GHz' (IF IDs with the center frequencies in range 30-45GHz; all channels)
                 spw='0:5~61' (IF ID 0; channels 5 to 61; all channels)
                 spw='3:10~20;50~60' (select multiple channel ranges within IF ID 3)
                 spw='3:10~20,4:0~30' (select different channel ranges for IF IDs 3 and 4)
                 spw='1~4;6:15~48' (for channels 15 through 48 for IF IDs 1,2,3,4 and 6)
        this selection is in addition to the other selections to data
    >>> spw expandable parameter
        restfreq -- the rest frequency
                    available type includes float, int, string, list of float, 
                    list of int, list of string, and list of dictionary. the 
                    default unit of restfreq in case of float, int, or string 
                    without unit is Hz. string input can be a value only 
                    (treated as Hz) or a value followed by unit for which 'GHz',
                    'MHz','kHz',and 'Hz' are available. 
                    a list can be used to set different rest frequencies for 
                    each IF. the length of list input must be number of IFs. 
                    dictionary input should be a pair of line name and 
                    frequency with keys of 'name' and 'value', respectively. 
                    values in the dictionary input follows the same manner as 
                    as for single float or string input. 
                example: 345.796
                         '1420MHz'
                         [345.8, 347.0, 356.7]
                         ['345.8MHz', '347.0MHz', '356.7MHz']
                         [{'name':'CO','value':345}]
        frame -- frequency reference frame
                options: 'LSRK', 'TOPO', 'LSRD', 'BARY', 'GALACTO', 'LGROUP', 'CMB'
                default: '' (keep current frame in data)
        doppler -- doppler convention (effective only when spw is in
                   velocity unit)
                options: 'RADIO', 'OPTICAL', 'Z', 'BETA', or 'GAMMA'
                default: '' (keep current doppler setting in data)
scan -- select data by scan numbers
        default: '' (use all scans)
        example: scan='21~23' (scan IDs 21,22,23)
        this selection is in addition to the other selections to data
pol -- select data by polarization IDs
        default: '' (use all polarizations)
        example: pol='0,1' (polarization IDs 0,1)
        this selection is in addition to the other selections to data
fitfunc -- function for fitting
        options: 'gauss' (Gaussian), 'lorentz' (Lorentzian)
        default: 'gauss'
fitmode -- mode for fitting
        options: 'auto', 'list', or 'interact'
        default: 'auto'
        example: 'auto' will use the linefinder to fit for lines
                        using the following parameters
                 'list' will use maskline to define regions to
                        fit for lines with nfit in each
                 'interact' allows adding and deleting mask 
                        regions by drawing rectangles on the plot 
                        with mouse. Draw a rectangle with LEFT-mouse 
                        to ADD the region to the mask and with RIGHT-mouse 
                        to DELETE the region. 

    >>> fitmode expandable parameters     
        thresh -- S/N threshold for linefinder. a single channel S/N ratio
                  above which the channel is considered to be a detection.
                default: 5
        min_nchan -- minimum number of consecutive channels required to
                     pass threshold
                       default: 3
        avg_limit -- channel averaging for broad lines. a number of
                     consecutive channels not greater than this parameter
                     can be averaged to search for broad lines.
                default: 4
        box_size -- running mean box size specified as a fraction
                    of the total spectrum length
                default: 0.2
        edge -- channels to drop at beginning and end of spectrum
                default: 0
                example: edge=[1000] drops 1000 channels at beginning AND end.
                         edge=[1000,500] drops 1000 from beginning and 500
                         from end

        Note: For bad baselines threshold should be increased,
        and avg_limit decreased (or even switched off completely by
        setting this parameter to 1) to avoid detecting baseline
        undulations instead of real lines.

nfit -- list of number of gaussian/lorentzian lines to fit in in maskline
        region (ignored when fitmode='auto')
        default: 0 (no fitting)
        example: nfit=[1] for single line in single region,
                 nfit=[2] for two lines in single region,
                 nfit=[1,1] for single lines in each of two regions, etc.
outfile -- name of output file
        default: no output fit file
        example: 'mysd.fit'
overwrite -- overwrite the output file if already exists
        options: (bool) True, False
        default: False
plotlevel -- control for plotting of results
        options: 0, 1, 2
        default: 0 (no plotting)
        example: plotlevel=0 no plotting
                 plotlevel=1 plots fit
                 plotlevel=2 plots fit and residual 
                 no hardcopy available for fitter
        WARNING: be careful plotting OTF data with lots of fields

-------
Returns
-------
a Python dictionary of line statistics
    keys: 'peak', 'cent', 'fwhm', 'nfit'
    example: each value is a list of lists with one list of
             2 entries [fitvalue,error] per component.
             e.g. xstat['peak']=[[234.9, 4.8],[234.2, 5.3]]
             for 2 components.

-----------
DESCRIPTION
-----------

Task sdfit is a basic line-fitter for single-dish spectra.
It assumes that the spectra have been calibrated in sdcal
or sdreduce.

Furthermore, it assumes that any selection of scans, IFs,
polarizations, and time and channel averaging/smoothing has
also already been done (in other sd tasks) as there are no controls
for these.  Note that you can use sdsave to do selection, writing
out a new scantable.

Note that multiple scans, IFs, and polarizations can in principle 
be handled, but we recommend that you use scan, field, spw, and pol
to give a single selection for each fit.

Currently, you can choose Gaussian or Lorentzian profile as a 
fitting model. 

For complicated spectra, sdfit does not do a good job of
"auto-guessing" the starting model for the fit.  We recommend
you use sd.fitter in the toolkit which has more options, such
as fixing components in the fit and supplying starting guesses
by hand.

-------
FITMODE
-------
As described in the parameter description section, sdfit implements
three fitting mode, 'auto', 'list', and 'interact'. Only difference
between these modes are a method to set initial guess for line fitting.
In 'auto' mode, initial guess is automatically set by executing line
finder. On the other hand, 'list' and 'interact' allow to set initial
guess manually. In these modes, only controllable parameter for the
guess is range of the line region and number of lines per region.
In 'list' mode, the user must give line region via spw parameter by using
ms selection syntax while number of lines per region can be specified
via nfit parameter. For example,
 
    spw = '17:1500~2500'
    nfit = [1]

will set line region between channels 1500 and 2500 for spw 17, and
indicate that there is only one line in this region. Specifying single
region with multiple line is also possible but is not recommended.
In 'interact' mode, spectral data to be fitted will be displayed
with pre-defined line region specified by spw parameter. The user is
able to customize line region interactively. 

--------------------
FLUX UNIT CONVERSION
--------------------
The task is able to convert flux unit between K and Jy. To do that,
fluxunit and its subparameter telescopeparam must be properly set.
The fluxunit should be 'Jy' or 'K' depending on what unit input data
is and what unit you want to convert. If given fluxunit is different
from the unit of input data, unit conversion is performed.
The telescopeparam is used to specify conversion factor. There are three
ways to specify telescopeparam: 1) set Jy/K conversion factor, 2) set
telescope diameter, D, and aperture efficiency, eta, separately, and
3) 'FIX' mode (only change the unit without converting spectral data).
If you give telescopeparam as a list, then if the list has a single float
it is assumed to be the gain in Jy/K (case 1), if two or more elements
they are assumed to be telescope diameter (m) and aperture efficiency
respectively (case 2).
See the above parameter description as well as note on 'FIX' mode below
for details.
  
There are two special cases that don't need telescopeparam for unit
conversion. Telescope name is obtained from the data.
1) ASAP (sd tool) recognizes the conversion factor (actually D and
   eta) for the "AT" telescopes, namely ATNF MOPRA telescope, until
   2004.
2) The task does know D and eta for GBT telescope.
If you wish to change the fluxunit, by leaving the sub-parameter
telescopeparam unset (telescopeparam=''), it will use internal telescope
parameters for flux conversion for the data from AT telescopes and it
will use an approximate aperture efficiency conversion for the GBT data.

Note that sdcal assumes that the fluxunit is set correctly in the data
already. If not, then set telescopeparam='FIX' and it will set the
default units to fluxunit without conversion.
Note also that, if the data in infile is an ms from GBT and the default
flux unit is missing, this task automatically fixes the default fluxunit
to 'K' before the conversion.

-------------------------------------
AVERAGING OF SPECTRA
-------------------------------------
Task sdfit has two averaging modes, i.e., time and polarization average.

When timeaverage=True, spectra are averaged over time for each IF
(spectral window), polarization, and beam, independently. Note that,
by default (scanaverage=False), timeaverage=True averages spectra
irrespective of scan IDs.
It is possible to average spectra separately for each scan ID by setting
a sub-parameter scanaverage=True.
For example, the combination of parameters: scan='0~2', timeaverage=True, and
scanaverage=False: averages spectra in scan ID 0 through 2 all together
                   to a spectrum,
scanaverage=True : averages spectra per scan ID and end up with three
                   spectra from scan 0, 1, and 2.

When polaverage=True, spectra are averaged over polarization for
each IF (spectral window) and beam. Note that, so far, time averaging is
automatically switched on when polaverage is set to True. This behavior
is not desirable and will be discarded in future.

-------
WARNING
-------
For the GBT raw SDFITS format data as input:
SDtasks are able to handle GBT raw SDFITS format data since the data
filler is available. However, the functionality is not well tested yet,
so that there may be unknown bugs.
  
        """
	if not hasattr(self, "__globals__") or self.__globals__ == None :
           self.__globals__=sys._getframe(len(inspect.stack())-1).f_globals
	#casac = self.__globals__['casac']
	casalog = self.__globals__['casalog']
	casa = self.__globals__['casa']
	#casalog = casac.casac.logsink()
        self.__globals__['__last_task'] = 'sdfit'
        self.__globals__['taskname'] = 'sdfit'
        ###
        self.__globals__['update_params'](func=self.__globals__['taskname'],printtext=False,ipython_globals=self.__globals__)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
        if type(self.__call__.func_defaults) is NoneType:
            function_signature_defaults={}
	else:
	    function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames[1:],self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       self.parameters[key] = keyVal
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
                      if len(keyVal) > 0 :
		         exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')
		      else :
		         exec('myparams[key] = ' + key + ' = {}')
	 
        else :
            print ''

            myparams['infile'] = infile = self.parameters['infile']
            myparams['antenna'] = antenna = self.parameters['antenna']
            myparams['fluxunit'] = fluxunit = self.parameters['fluxunit']
            myparams['telescopeparam'] = telescopeparam = self.parameters['telescopeparam']
            myparams['field'] = field = self.parameters['field']
            myparams['spw'] = spw = self.parameters['spw']
            myparams['restfreq'] = restfreq = self.parameters['restfreq']
            myparams['frame'] = frame = self.parameters['frame']
            myparams['doppler'] = doppler = self.parameters['doppler']
            myparams['scan'] = scan = self.parameters['scan']
            myparams['pol'] = pol = self.parameters['pol']
            myparams['timeaverage'] = timeaverage = self.parameters['timeaverage']
            myparams['tweight'] = tweight = self.parameters['tweight']
            myparams['scanaverage'] = scanaverage = self.parameters['scanaverage']
            myparams['polaverage'] = polaverage = self.parameters['polaverage']
            myparams['pweight'] = pweight = self.parameters['pweight']
            myparams['fitfunc'] = fitfunc = self.parameters['fitfunc']
            myparams['fitmode'] = fitmode = self.parameters['fitmode']
            myparams['nfit'] = nfit = self.parameters['nfit']
            myparams['thresh'] = thresh = self.parameters['thresh']
            myparams['min_nchan'] = min_nchan = self.parameters['min_nchan']
            myparams['avg_limit'] = avg_limit = self.parameters['avg_limit']
            myparams['box_size'] = box_size = self.parameters['box_size']
            myparams['edge'] = edge = self.parameters['edge']
            myparams['outfile'] = outfile = self.parameters['outfile']
            myparams['overwrite'] = overwrite = self.parameters['overwrite']
            myparams['plotlevel'] = plotlevel = self.parameters['plotlevel']

        if type(nfit)==int: nfit=[nfit]
        if type(edge)==int: edge=[edge]

	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['infile'] = infile
        mytmp['antenna'] = antenna
        mytmp['fluxunit'] = fluxunit
        mytmp['telescopeparam'] = telescopeparam
        mytmp['field'] = field
        mytmp['spw'] = spw
        mytmp['restfreq'] = restfreq
        mytmp['frame'] = frame
        mytmp['doppler'] = doppler
        mytmp['scan'] = scan
        mytmp['pol'] = pol
        mytmp['timeaverage'] = timeaverage
        mytmp['tweight'] = tweight
        mytmp['scanaverage'] = scanaverage
        mytmp['polaverage'] = polaverage
        mytmp['pweight'] = pweight
        mytmp['fitfunc'] = fitfunc
        mytmp['fitmode'] = fitmode
        mytmp['nfit'] = nfit
        mytmp['thresh'] = thresh
        mytmp['min_nchan'] = min_nchan
        mytmp['avg_limit'] = avg_limit
        mytmp['box_size'] = box_size
        mytmp['edge'] = edge
        mytmp['outfile'] = outfile
        mytmp['overwrite'] = overwrite
        mytmp['plotlevel'] = plotlevel
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
	trec = casac.casac.utils().torecord(pathname+'sdfit.xml')

        casalog.origin('sdfit')
	try :
          #if not trec.has_key('sdfit') or not casac.casac.utils().verify(mytmp, trec['sdfit']) :
	    #return False

          casac.casac.utils().verify(mytmp, trec['sdfit'], True)
          scriptstr=['']
          saveinputs = self.__globals__['saveinputs']
          if type(self.__call__.func_defaults) is NoneType:
              saveinputs=''
          else:
              saveinputs('sdfit', 'sdfit.last', myparams, self.__globals__,scriptstr=scriptstr)
          tname = 'sdfit'
          spaces = ' '*(18-len(tname))
          casalog.post('\n##########################################'+
                       '\n##### Begin Task: ' + tname + spaces + ' #####')
          if type(self.__call__.func_defaults) is NoneType:
              casalog.post(scriptstr[0]+'\n', 'INFO')
          else :
              casalog.post(scriptstr[1][1:]+'\n', 'INFO')
          result = sdfit(infile, antenna, fluxunit, telescopeparam, field, spw, restfreq, frame, doppler, scan, pol, timeaverage, tweight, scanaverage, polaverage, pweight, fitfunc, fitmode, nfit, thresh, min_nchan, avg_limit, box_size, edge, outfile, overwrite, plotlevel)
          casalog.post('##### End Task: ' + tname + '  ' + spaces + ' #####'+
                       '\n##########################################')

	except Exception, instance:
          if(self.__globals__.has_key('__rethrow_casa_exceptions') and self.__globals__['__rethrow_casa_exceptions']) :
             raise
          else :
             #print '**** Error **** ',instance
	     tname = 'sdfit'
             casalog.post('An error occurred running task '+tname+'.', 'ERROR')
             pass

        gc.collect()
        return result
#
#
#
    def paramgui(self, useGlobals=True, ipython_globals=None):
        """
        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
        """
        import paramgui
	if not hasattr(self, "__globals__") or self.__globals__ == None :
           self.__globals__=sys._getframe(len(inspect.stack())-1).f_globals

        if useGlobals:
	    if ipython_globals == None:
                myf=self.__globals__
            else:
                myf=ipython_globals

            paramgui.setGlobals(myf)
        else:
            paramgui.setGlobals({})

        paramgui.runTask('sdfit', myf['_ip'])
        paramgui.setGlobals({})

#
#
#
    def defaults(self, param=None, ipython_globals=None, paramvalue=None, subparam=None):
	if not hasattr(self, "__globals__") or self.__globals__ == None :
           self.__globals__=sys._getframe(len(inspect.stack())-1).f_globals
        if ipython_globals == None:
            myf=self.__globals__
        else:
            myf=ipython_globals

        a = odict()
        a['infile']  = ''
        a['antenna']  = 0
        a['fluxunit']  = ''
        a['field']  = ''
        a['spw']  = ''
        a['scan']  = ''
        a['pol']  = ''
        a['timeaverage']  = False
        a['polaverage']  = False
        a['fitfunc']  = 'gauss'
        a['fitmode']  = 'auto'
        a['nfit']  = []
        a['outfile']  = ''
        a['overwrite']  = False
        a['plotlevel']  = 0

        a['fluxunit'] = {
                    0:{'value':''}, 
                    1:odict([{'value':'K'}, {'telescopeparam':''}]), 
                    2:odict([{'value':'k'}, {'telescopeparam':''}]), 
                    3:odict([{'value':'Jy'}, {'telescopeparam':''}]), 
                    4:odict([{'value':'jy'}, {'telescopeparam':''}])}
        a['spw'] = {
                    0:odict([{'notvalue':''}, {'restfreq':''}, {'frame':''}, {'doppler':''}])}
        a['timeaverage'] = {
                    0:{'value':False}, 
                    1:odict([{'value':True}, {'tweight':'tintsys'}, {'scanaverage':False}])}
        a['polaverage'] = {
                    0:{'value':False}, 
                    1:odict([{'value':True}, {'pweight':'tsys'}])}
        a['fitmode'] = {
                    0:odict([{'value':'auto'}, {'thresh':5.0}, {'min_nchan':3}, {'avg_limit':4}, {'box_size':0.2}, {'edge':[0]}]), 
                    1:{'value':'list'}, 
                    2:{'value':'interact'}}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
            if(paramvalue==None and subparam==None):
               if(a.has_key(param)):
                  return a[param]
               else:
                  return self.itsdefault(param)
            else:
               retval=a[param]
               if(type(a[param])==dict):
                  for k in range(len(a[param])):
                     valornotval='value'
                     if(a[param][k].has_key('notvalue')):
                        valornotval='notvalue'
                     if((a[param][k][valornotval])==paramvalue):
                        retval=a[param][k].copy()
                        retval.pop(valornotval)
                        if(subparam != None):
                           if(retval.has_key(subparam)):
                              retval=retval[subparam]
                           else:
                              retval=self.itsdefault(subparam)
		     else:
                        retval=self.itsdefault(subparam)
               return retval


#
#
    def check_params(self, param=None, value=None, ipython_globals=None):
      if ipython_globals == None:
          myf=self.__globals__
      else:
          myf=ipython_globals
#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value
#
#
    def description(self, key='sdfit', subkey=None):
        desc={'sdfit': 'Fit a spectral line',
               'infile': 'name of input SD dataset',
               'antenna': 'select an antenna name or ID, e.g. \'PM03\' (only effective for MS input)',
               'fluxunit': 'units of the flux [\'K\' or \'Jy\'] (\'\'=current) ',
               'telescopeparam': 'parameters of telescope for flux conversion (see examples in help)',
               'field': 'select data by field IDs and names, e.g. \'3C2*\' (\'\'=all)',
               'spw': 'select data by IF IDs (spectral windows), e.g. \'3,5,7\' (\'\'=all)',
               'restfreq': 'the rest frequency, e.g. \'1.41GHz\' (default unit: Hz) (see examples in help)',
               'frame': 'frequency reference frame [\'LSRK\', \'TOPO\', \'LSRD\', \'BARY\', \'GALACTO\', \'LGROUP\', or \'CMB\'] (\'\'=current)',
               'doppler': 'doppler convention [\'RADIO\', \'OPTICAL\', \'Z\', \'BETA\', or \'GAMMA\'] (\'\'=current).',
               'scan': 'select data by scan numbers, e.g. \'21~23\' (\'\'=all)',
               'pol': 'select data by polarization IDs, e.g. \'0,1\' (\'\'=all)',
               'timeaverage': 'average spectra over time [True, False] (see examples in help)',
               'tweight': 'weighting for time averaging [\'tintsys\', \'tsys\', \'tint\', \'var\', or \'median\']',
               'scanaverage': 'average spectra within a scan number [True, False] (see examples in help)',
               'polaverage': 'average spectra over polarizations [True, False]',
               'pweight': 'weighting for polarization averaging [\'tsys\' or \'var\']',
               'fitfunc': 'function for fitting [\'gauss\', \'lorentz\']',
               'fitmode': 'mode for fitting [\'auto\', \'list\', or \'interact\']',
               'nfit': 'list of number of gaussian/lorentzian lines to fit in in maskline region (ignored when fitmode="auto")',
               'thresh': 'S/N threshold for linefinder',
               'min_nchan': 'minimum number of consecutive channels for linefinder',
               'avg_limit': 'channel averaging for broad lines',
               'box_size': 'running mean box size',
               'edge': 'channels to drop at beginning and end of spectrum',
               'outfile': 'name of output file (See a WARNING in help)',
               'overwrite': 'overwrite the output file if already exists [True, False]',
               'plotlevel': 'control for plotting of results (see examples in help)',

              }

#
# Set subfields defaults if needed
#

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['infile']  = ''
        a['antenna']  = 0
        a['fluxunit']  = ''
        a['telescopeparam']  = ''
        a['field']  = ''
        a['spw']  = ''
        a['restfreq']  = ''
        a['frame']  = ''
        a['doppler']  = ''
        a['scan']  = ''
        a['pol']  = ''
        a['timeaverage']  = False
        a['tweight']  = 'tintsys'
        a['scanaverage']  = False
        a['polaverage']  = False
        a['pweight']  = 'tsys'
        a['fitfunc']  = 'gauss'
        a['fitmode']  = 'auto'
        a['nfit']  = []
        a['thresh']  = 5.0
        a['min_nchan']  = 3
        a['avg_limit']  = 4
        a['box_size']  = 0.2
        a['edge']  = [0]
        a['outfile']  = ''
        a['overwrite']  = False
        a['plotlevel']  = 0

        #a = sys._getframe(len(inspect.stack())-1).f_globals

        if self.parameters['fluxunit']  == 'K':
            a['telescopeparam'] = ''

        if self.parameters['fluxunit']  == 'k':
            a['telescopeparam'] = ''

        if self.parameters['fluxunit']  == 'Jy':
            a['telescopeparam'] = ''

        if self.parameters['fluxunit']  == 'jy':
            a['telescopeparam'] = ''

        if self.parameters['spw']  != '':
            a['restfreq'] = ''
            a['frame'] = ''
            a['doppler'] = ''

        if self.parameters['timeaverage']  == True:
            a['tweight'] = 'tintsys'
            a['scanaverage'] = False

        if self.parameters['polaverage']  == True:
            a['pweight'] = 'tsys'

        if self.parameters['fitmode']  == 'auto':
            a['thresh'] = 5.0
            a['min_nchan'] = 3
            a['avg_limit'] = 4
            a['box_size'] = 0.2
            a['edge'] = [0]

        if a.has_key(paramname) :
	      return a[paramname]
sdfit_cli = sdfit_cli_()
