#
# This file was generated using xslt from its XML file
#
# Copyright 2009, Associated Universities Inc., Washington DC
#
import sys
import os
from  casac import *
import string
from taskinit import casalog
from taskinit import xmlpath
#from taskmanager import tm
import task_sdscale
def sdscale(infile='', antenna=0, factor=1.0, scaletsys=True, outfile='', overwrite=False, verbose=True):

        """Scale the sd spectra
------------------
Keyword arguments
------------------
infile -- name of input SD dataset
antenna -- select an antenna name or ID
        default: 0
        example: 'PM03'
        NOTE this parameter is effective only for MS input
factor -- scaling factor. float, one- or two-dimensional float list,
          or filename storing scaling factor are acceptable
       default: 1.0 (no scaling)
       example: see description below
scaletsys -- scaling of associated Tsys
          options: (bool) True, False
          default: True
outfile -- name of output file
        default: outfile='' (<infile>_scaled<factor>)
        example: 'scaled.asap'
overwrite -- overwrite the output file if already exists
        options: (bool) True,False
        default: False
        NOTE this parameter is ignored when outform='ASCII'
verbose -- Print verbose log messages. If True, Tsys values before
           (and after) scaling are printed to logger.
          options: (bool) True, False
          default: True

-----------
DESCRIPTION
-----------
Task sdscale performs scaling of single-dish spectra.
Associated Tsys is also scaled if scaletsys is True.
Tsys informations are written in the casalogger and they are 
automatically stored in 'casapy.log'. 
The infile can be any of ASAP, MS, SDFITS, or RPFITS format.
If outfile name is given or outfile=''(default), the scaled data 
is written to a new file with the same format as the input data 
(Note: in case of the RPFITS format input data, it will be written 
to SDFITS format).

The scaling factor, factor, accepts both scalar type and list 
type value. The list must be one or two dimensional. If factor is 
one dimensional, its length must coincide with a number of spectral 
channel. If factor is two dimensional, its shape must be 
(n, 1) or (n, m), where n is a number of spectrum, while m is a 
number of channel for each spcetum. m can be variable for each 
spectrum. In addition, the factor can be an ASCII filename that 
stores a space-separated list of scaling factor consisting of 
adequate number of rows and columns. For example, if the contents 
of input ASCII file is shown as, 

   0.5 0.3 0.2
   1.0 0.2 0.9

it is interpreted as a list [[0.5, 0.3, 0.2],[1.0, 0.2, 0.9]]. 

-------
WARNING
-------
For the GBT raw SDFITS format data as input:
SDtasks are able to handle GBT raw SDFITS format data since the 
data filler is available. However, the functionality is not well 
tested yet, so that there may be unknown bugs.  

  
        """

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['infile'] = infile
        mytmp['antenna'] = antenna
        mytmp['factor'] = factor
        mytmp['scaletsys'] = scaletsys
        mytmp['outfile'] = outfile
        mytmp['overwrite'] = overwrite
        mytmp['verbose'] = verbose
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
	trec = casac.utils().torecord(pathname+'sdscale.xml')

        casalog.origin('sdscale')
        if trec.has_key('sdscale') and casac.utils().verify(mytmp, trec['sdscale']) :
	    result = task_sdscale.sdscale(infile, antenna, factor, scaletsys, outfile, overwrite, verbose)

	else :
	  result = False
        return result
